@@ -55,7 +55,7 @@ class TicketEmailParser(object):
             if part.get_content_type() == 'text/plain':
                 tkt['description'] = part.get_payload(decode=1).strip()
 
-        if tkt.get('description'):
+        if tkt.values.get('description'):
             tkt.insert()
 
 if __name__ == '__main__':
