@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2005 Edgewall Software
-# Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
+# Copyright (C) 2003-2008 Edgewall Software
+# Copyright (C) 2003-2007 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -15,13 +15,23 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 
 import os
+try:
+    import threading
+except ImportError:
+    import dummy_threading as threading
+import setuptools
+import sys
+from urlparse import urlsplit
 
-from trac import db_default, util
+from trac import db_default
 from trac.config import *
 from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracError
 from trac.db import DatabaseManager
+from trac.util import get_pkginfo
+from trac.util.translation import _
 from trac.versioncontrol import RepositoryManager
+from trac.web.href import Href
 
 __all__ = ['Environment', 'IEnvironmentSetupParticipant', 'open_environment']
 
@@ -57,18 +67,38 @@ class Environment(Component, ComponentManager):
     things:
      * a configuration file.
      * an SQLite database (stores tickets, wiki pages...)
-     * Project specific templates and wiki macros.
+     * Project specific templates and plugins.
      * wiki and ticket attachments.
     """   
     setup_participants = ExtensionPoint(IEnvironmentSetupParticipant)
 
-    base_url = Option('trac', 'base_url', '',
-        """Base URL of the Trac deployment.
+    shared_plugins_dir = PathOption('inherit', 'plugins_dir', '',
+        """Path of the directory containing additional plugins.
         
-        In most configurations, Trac will automatically reconstruct the URL
-        that is used to access it automatically. However, in more complex
-        setups, usually involving running Trac behind a HTTP proxy, you may
-        need to use this option to force Trac to use the correct URL.""")
+        Plugins in that directory are loaded in addition to those in the
+        directory of the environment `plugins`, with this one taking 
+        precedence.
+        
+        (''since 0.11'')""")
+
+    base_url = Option('trac', 'base_url', '',
+        """Reference URL for the Trac deployment.
+        
+        This is the base URL that will be used when producing documents that
+        will be used outside of the web browsing context, like for example
+        when inserting URLs pointing to Trac resources in notification
+        e-mails.""")
+
+    base_url_for_redirect = BoolOption('trac', 'use_base_url_for_redirect',
+            False, 
+        """Optionally use `[trac] base_url` for redirects.
+        
+        In some configurations, usually involving running Trac behind a HTTP
+        proxy, Trac can't automatically reconstruct the URL that is used to
+        access it. You may need to use this option to force Trac to use the
+        `base_url` setting also for redirects. This introduces the obvious
+        limitation that this environment will only be usable when accessible
+        from that URL, as redirects are frequently used. ''(since 0.10.5)''""")
 
     project_name = Option('project', 'name', 'My Project',
         """Name of the project.""")
@@ -76,8 +106,12 @@ class Environment(Component, ComponentManager):
     project_description = Option('project', 'descr', 'My example project',
         """Short description of the project.""")
 
-    project_url = Option('project', 'url', 'http://example.org/',
-        """URL of the main project web site.""")
+    project_url = Option('project', 'url', '',
+        """URL of the main project web site, usually the website in which
+        the `base_url` resides.""")
+
+    project_admin = Option('project', 'admin', '',
+        """E-Mail address of the project's administrator.""")
 
     project_footer = Option('project', 'footer',
                             'Visit the Trac open source project at<br />'
@@ -120,7 +154,7 @@ class Environment(Component, ComponentManager):
          Example:
          ($(thread)d) Trac[$(basename)s:$(module)s] $(levelname)s: $(message)s
 
-         (since 0.11)""")
+         (since 0.10.5)""")
 
     def __init__(self, path, create=False, options=[]):
         """Initialize the Trac environment.
@@ -138,8 +172,17 @@ class Environment(Component, ComponentManager):
         self.setup_config(load_defaults=create)
         self.setup_log()
 
+        from trac import core, __version__ as VERSION
+        self.systeminfo = [
+            ('Trac', get_pkginfo(core).get('version', VERSION)),
+            ('Python', sys.version),
+            ('setuptools', setuptools.__version__),
+            ]
+        self._href = self._abs_href = None
+
         from trac.loader import load_components
-        load_components(self)
+        plugins_dir = self.config.get('inherit', 'plugins_dir')
+        load_components(self, plugins_dir and (plugins_dir,))
 
         if create:
             self.create(options)
@@ -179,6 +222,17 @@ class Environment(Component, ComponentManager):
         for pattern, enabled in rules:
             if component_name == pattern or pattern.endswith('*') \
                     and component_name.startswith(pattern[:-1]):
+                # Disable the pre-0.11 WebAdmin plugin
+                # Please note that there's no recommendation to uninstall the
+                # plugin because doing so would obviously break the backwards
+                # compatibility that the new integration administration
+                # interface tries to provide for old WebAdmin extensions
+                if component_name.startswith('webadmin.'):
+                    self.log.info('The legacy TracWebAdmin plugin has been '
+                                  'automatically disabled, and the integrated '
+                                  'administration interface will be used '
+                                  'instead.')
+                    return False
                 return enabled
 
         # versioncontrol components are enabled if the repository is configured
@@ -217,10 +271,15 @@ class Environment(Component, ComponentManager):
 
     def create(self, options=[]):
         """Create the basic directory structure of the environment, initialize
-        the database and populate the configuration file with default values."""
+        the database and populate the configuration file with default values.
+
+        If options contains ('inherit', 'file'), default values will not be
+        loaded; they are expected to be provided by that file or other options.
+        """
         def _create_file(fname, data=None):
             fd = open(fname, 'w')
-            if data: fd.write(data)
+            if data:
+                fd.write(data)
             fd.close()
 
         # Create the directory structure
@@ -229,7 +288,6 @@ class Environment(Component, ComponentManager):
         os.mkdir(self.get_log_dir())
         os.mkdir(self.get_htdocs_dir())
         os.mkdir(os.path.join(self.path, 'plugins'))
-        os.mkdir(os.path.join(self.path, 'wiki-macros'))
 
         # Create a few files
         _create_file(os.path.join(self.path, 'VERSION'),
@@ -241,20 +299,33 @@ class Environment(Component, ComponentManager):
         # Setup the default configuration
         os.mkdir(os.path.join(self.path, 'conf'))
         _create_file(os.path.join(self.path, 'conf', 'trac.ini'))
-        self.setup_config(load_defaults=True)
+        skip_defaults = options and ('inherit', 'file') in [(section, option) \
+                for (section, option, value) in options]
+        self.setup_config(load_defaults=not skip_defaults)
         for section, name, value in options:
             self.config.set(section, name, value)
         self.config.save()
+        self.config.parse_if_needed() # Full reload to get 'inherit' working
 
         # Create the database
         DatabaseManager(self).init_db()
 
-    def get_version(self, db=None):
-        """Return the current version of the database."""
+    def get_version(self, db=None, initial=False):
+        """Return the current version of the database.
+        If the optional argument `initial` is set to `True`, the version
+        of the database used at the time of creation will be returned.
+
+        In practice, for database created before 0.11, this will return `False`
+        which is "older" than any db version number.
+
+        :since 0.11:
+        """
         if not db:
             db = self.get_db_cnx()
         cursor = db.cursor()
-        cursor.execute("SELECT value FROM system WHERE name='database_version'")
+        cursor.execute("SELECT value FROM system "
+                       "WHERE name='%sdatabase_version'" %
+                       (initial and 'initial_' or ''))
         row = cursor.fetchone()
         return row and int(row[0])
 
@@ -262,9 +333,9 @@ class Environment(Component, ComponentManager):
         """Load the configuration file."""
         self.config = Configuration(os.path.join(self.path, 'conf', 'trac.ini'))
         if load_defaults:
-            for section, default_options in self.config.defaults().iteritems():
-                for name, value in default_options.iteritems():
-                    if self.config.has_site_option(section, name):
+            for section, default_options in self.config.defaults().items():
+                for name, value in default_options.items():
+                    if self.config.parent and name in self.config.parent[section]:
                         value = None
                     self.config.set(section, name, value)
 
@@ -330,7 +401,7 @@ class Environment(Component, ComponentManager):
 
         db_str = self.config.get('trac', 'database')
         if not db_str.startswith('sqlite:'):
-            raise EnvironmentError('Can only backup sqlite databases')
+            raise TracError(_('Can only backup sqlite databases'))
         db_name = os.path.join(self.path, db_str[7:])
         if not dest:
             dest = '%s.%i.bak' % (db_name, self.get_version())
@@ -376,6 +447,23 @@ class Environment(Component, ComponentManager):
 
         return True
 
+    def _get_href(self):
+        if not self._href:
+            self._href = Href(urlsplit(self.abs_href.base)[2])
+        return self._href
+    href = property(_get_href, 'The application root path')
+
+    def _get_abs_href(self):
+        if not self._abs_href:
+            if not self.base_url:
+                self.log.warn('base_url option not set in configuration, '
+                              'generated links may be incorrect')
+                self._abs_href = Href('')
+            else:
+                self._abs_href = Href(self.base_url)
+        return self._abs_href
+    abs_href = property(_get_abs_href, 'The application URL')
+
 
 class EnvironmentSetup(Component):
     implements(IEnvironmentSetupParticipant)
@@ -398,7 +486,7 @@ class EnvironmentSetup(Component):
         if dbver == db_default.db_version:
             return False
         elif dbver > db_default.db_version:
-            raise TracError, 'Database newer than Trac version'
+            raise TracError(_('Database newer than Trac version'))
         return True
 
     def upgrade_environment(self, db):
@@ -410,8 +498,8 @@ class EnvironmentSetup(Component):
                 upgrades = __import__('upgrades', globals(), locals(), [name])
                 script = getattr(upgrades, name)
             except AttributeError:
-                err = 'No upgrade module for version %i (%s.py)' % (i, name)
-                raise TracError, err
+                raise TracError(_('No upgrade module for version %(num)i '
+                                  '(%(version)s.py)', num=i, version=name))
             script.do_upgrade(self.env, i, cursor)
         cursor.execute("UPDATE system SET value=%s WHERE "
                        "name='database_version'", (db_default.db_version,))
@@ -422,20 +510,13 @@ class EnvironmentSetup(Component):
     # Internal methods
 
     def _update_sample_config(self):
-        from ConfigParser import ConfigParser
-        config = ConfigParser()
-        for section, options in self.config.defaults().items():
-            config.add_section(section)
-            for name, value in options.items():
-                config.set(section, name, value)
         filename = os.path.join(self.env.path, 'conf', 'trac.ini.sample')
+        config = Configuration(filename)
+        for section, default_options in config.defaults().iteritems():
+            for name, value in default_options.iteritems():
+                config.set(section, name, value)
         try:
-            fileobj = file(filename, 'w')
-            try:
-                config.write(fileobj)
-                fileobj.close()
-            finally:
-                fileobj.close()
+            config.save()
             self.log.info('Wrote sample configuration file with the new '
                           'settings and their default values: %s',
                           filename)
@@ -444,23 +525,58 @@ class EnvironmentSetup(Component):
                           exc_info=True)
 
 
-def open_environment(env_path=None):
+env_cache = {}
+env_cache_lock = threading.Lock()
+
+def open_environment(env_path=None, use_cache=False):
     """Open an existing environment object, and verify that the database is up
     to date.
 
-    @param: env_path absolute path to the environment directory; if ommitted,
-            the value of the `TRAC_ENV` environment variable is used
+    @param env_path: absolute path to the environment directory; if ommitted,
+                     the value of the `TRAC_ENV` environment variable is used
+    @param use_cache: whether the environment should be cached for subsequent
+                      invocations of this function
     @return: the `Environment` object
     """
+    global env_cache, env_cache_lock
+
     if not env_path:
         env_path = os.getenv('TRAC_ENV')
     if not env_path:
-        raise TracError, 'Missing environment variable "TRAC_ENV". Trac ' \
-                         'requires this variable to point to a valid Trac ' \
-                         'environment.'
+        raise TracError(_('Missing environment variable "TRAC_ENV". '
+                          'Trac requires this variable to point to a valid '
+                          'Trac environment.'))
+
+    if use_cache:
+        env_cache_lock.acquire()
+        try:
+            env = env_cache.get(env_path)
+            if env and env.config.parse_if_needed():
+                # The environment configuration has changed, so shut it down
+                # and remove it from the cache so that it gets reinitialized
+                env.log.info('Reloading environment due to configuration '
+                             'change')
+                env.shutdown()
+                if hasattr(env.log, '_trac_handler'):
+                    hdlr = env.log._trac_handler
+                    env.log.removeHandler(hdlr)
+                    hdlr.close()
+                del env_cache[env_path]
+                env = None
+            if env is None:
+                env = env_cache.setdefault(env_path, open_environment(env_path))
+        finally:
+            env_cache_lock.release()
+    else:
+        env = Environment(env_path)
+        needs_upgrade = False
+        try:
+            needs_upgrade = env.needs_upgrade()
+        except Exception, e: # e.g. no database connection
+            env.log.exception(e)
+        if needs_upgrade:
+            raise TracError(_('The Trac Environment needs to be upgraded.\n\n'
+                              'Run "trac-admin %(path)s upgrade"',
+                              path=env_path))
 
-    env = Environment(env_path)
-    if env.needs_upgrade():
-        raise TracError, 'The Trac Environment needs to be upgraded. Run ' \
-                         'trac-admin %s upgrade"' % env_path
     return env
