@@ -1,16 +1,16 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2005 Edgewall Software
+# Copyright (C) 2004-2008 Edgewall Software
 # Copyright (C) 2004 Oliver Rutherfurd
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Daniel Lundin
 #         Oliver Rutherfurd (initial implementation)
@@ -18,7 +18,7 @@
 #
 # Trac support for reStructured Text, including a custom 'trac' directive
 #
-# 'trac' directive code by Oliver Rutherfurd.
+# 'trac' directive code by Oliver Rutherfurd, overhauled by cboos.
 #
 # Inserts `reference` nodes for TracLinks into the document tree.
 
@@ -26,42 +26,22 @@ __docformat__ = 'reStructuredText'
 
 from distutils.version import StrictVersion
 import re
+try:
+    from docutils import nodes
+    from docutils.core import publish_parts
+    from docutils.parsers import rst
+    from docutils import __version__
+    has_docutils = True
+except ImportError:
+    has_docutils = False
 
 from trac.core import *
-from trac.mimeview.api import IHTMLPreviewRenderer
+from trac.mimeview.api import IHTMLPreviewRenderer, content_to_unicode
+from trac.util.html import Element, Markup
+from trac.util.translation import _
 from trac.web.href import Href
-from trac.wiki.formatter import WikiProcessor
-from trac.wiki import WikiSystem
-
-WIKI_LINK = re.compile(r'(?:wiki:)?(.+)')
-TICKET_LINK = re.compile(r'(?:#(\d+))|(?:ticket:(\d+))')
-REPORT_LINK = re.compile(r'(?:{(\d+)})|(?:report:(\d+))')
-CHANGESET_LINK = re.compile(r'(?:\[(\d+)\])|(?:changeset:(\d+))')
-FILE_LINK = re.compile(r'(?:browser|repos|source):([^#]+)#?(.*)')
-
-def _wikipage(href, args):
-    return href.wiki(args[0])
-
-def _ticket(href, args):
-    return href.ticket(args[0])
-
-def _report(href, args):
-    return href.report(args[0])
-
-def _changeset(href, args):
-    return href.changeset(int(args[0]))
-
-def _browser(href, args):
-    path = args[0]
-    rev = len(args) == 2 and args[1] or ''
-    return href.browser(path, rev=rev)
-
-# TracLink REs and callback functions
-LINKS = [(TICKET_LINK, _ticket),
-         (REPORT_LINK, _report),
-         (CHANGESET_LINK, _changeset),
-         (FILE_LINK, _browser),
-         (WIKI_LINK, _wikipage)]
+from trac.wiki.api import WikiSystem
+from trac.wiki.formatter import WikiProcessor, Formatter, extract_link
 
 class ReStructuredTextRenderer(Component):
     """
@@ -74,37 +54,38 @@ class ReStructuredTextRenderer(Component):
             return 8
         return 0
 
-    def render(self, req, mimetype, content, filename=None, rev=None):
-        try:
-            from docutils import nodes
-            from docutils.core import publish_string
-            from docutils.parsers import rst
-            from docutils import __version__
-        except ImportError:
-            raise TracError, 'Docutils not found'
-        if StrictVersion(__version__) < StrictVersion('0.3.3'):
-            raise TracError, 'Docutils version >= %s required, %s found' \
-                             % ('0.3.3', __version__)
+    def render(self, context, mimetype, content, filename=None, rev=None):
+        global has_docutils
+        if not has_docutils:
+            raise TracError(_('Docutils not found'))
+        if StrictVersion(__version__) < StrictVersion('0.3.9'):
+            raise TracError(_('Docutils version >= %(version)s required, '
+                              '%(found)s found', version='0.3.9',
+                              found=__version__))
 
-        def trac_get_reference(rawtext, link, text):
-            for (pattern, function) in LINKS:
-                m = pattern.match(link)
-                if m:
-                    g = filter(None, m.groups())
-                    missing = 0
-                    if not text:
-                        text = g[0]
-                    if pattern == WIKI_LINK:
-                        pagename = re.search(r'^[^\#]+',g[0])
-                        if not WikiSystem(self.env).has_page(pagename.group()):
-                            missing = 1
-                            text = text + "?"
-                    uri = function(self.env.href, g)
-                    reference = nodes.reference(rawtext, text)
-                    reference['refuri']= uri
-                    if missing:
-                        reference.set_class('missing')
-                    return reference
+        def trac_get_reference(rawtext, target, text):
+            fulltext = text and target+' '+text or target
+            link = extract_link(self.env, context, fulltext)
+            uri = None
+            missing = False
+            if isinstance(link, Element):
+                linktext = Markup(link).striptags()
+                # the following is a bit hackish, but it takes into account:
+                #  - an eventual trailing '?' for missing wiki pages
+                #  - space eventually introduced due to split_page_names option
+                if linktext.rstrip('?').replace(' ', '') != target:
+                    text = linktext
+                uri = link.attrib.get('href', '')
+                missing = 'missing' in link.attrib.get('class', '')
+            else:
+                uri = context.href.wiki(target)
+                missing = not WikiSystem(self.env).has_page(target)
+            if uri:                    
+                reference = nodes.reference(rawtext, text or target)
+                reference['refuri']= uri
+                if missing:
+                    reference.set_class('missing')
+                return reference
             return None
 
         def trac(name, arguments, options, content, lineno,
@@ -117,18 +98,13 @@ class ReStructuredTextRenderer(Component):
 
               .. trac:: target [text]
 
-            ``target`` may be one of the following:
-
-              * For wiki: ``WikiName`` or ``wiki:WikiName``
-               * For tickets: ``#1`` or ``ticket:1``
-              * For reports: ``{1}`` or ``report:1``
-              * For changesets: ``[1]`` or ``changeset:1``
-              * For files: ``source:trunk/COPYING``
+            ``target`` may be any `TracLink`_, provided it doesn't
+            embed a space character (e.g. wiki:"..." notation won't work).
 
             ``[text]`` is optional.  If not given, ``target`` is
             used as the reference text.
 
-            .. _TracLink: http://projects.edgewall.com/trac/wiki/TracLinks
+            .. _TracLink: http://trac.edgewall.org/wiki/TracLinks
             """
             link = arguments[0]
             if len(arguments) == 2:
@@ -172,19 +148,21 @@ class ReStructuredTextRenderer(Component):
 
         # The code_block could is taken from the leo plugin rst2
         def code_formatter(language, text):
-            processor = WikiProcessor(self.env, language)
-            html = processor.process(req, text)
+            processor = WikiProcessor(Formatter(self.env, context), language)
+            html = processor.process(text)
             raw = nodes.raw('', html, format='html')
             return raw
         
         def code_role(name, rawtext, text, lineno, inliner, options={},
                       content=[]):
-            args  = text.split(":",1)
-            language = args[0]
-            if len(args)==2:
-                text = args[1]
-            else:
-                text = ""
+            language = options.get('language')
+            if not language:
+                args  = text.split(':', 1)
+                language = args[0]
+                if len(args) == 2:
+                    text = args[1]
+                else:
+                    text = ''
             reference = code_formatter(language, text)
             return [reference], []
         
@@ -210,7 +188,7 @@ class ReStructuredTextRenderer(Component):
             0) # True if final argument may contain whitespace.
     
         # A mapping from option name to conversion function.
-        code_block.options = {
+        code_role.options = code_block.options = {
             'language' :
             rst.directives.unchanged # Return the text argument, unchanged
         }
@@ -221,7 +199,9 @@ class ReStructuredTextRenderer(Component):
 
         _inliner = rst.states.Inliner()
         _parser = rst.Parser(inliner=_inliner)
-
-        html = publish_string(content, writer_name='html', parser=_parser,
-                              settings_overrides={'halt_level': 6})
-        return html[html.find('<body>') + 6:html.find('</body>')].strip()
+        content = content_to_unicode(self.env, content, mimetype)
+        parts = publish_parts(content, writer_name='html', parser=_parser,
+                              settings_overrides={'halt_level': 6, 
+                                                  'file_insertion_enabled': 0, 
+                                                  'raw_enabled': 0})
+        return parts['html_body']
