@@ -1,11 +1,13 @@
-from trac.mimeview.tests import api, php
+from trac.mimeview.tests import api, patch, php, pygments
 
 import unittest
 
 def suite():
     suite = unittest.TestSuite()
     suite.addTest(api.suite())
+    suite.addTest(patch.suite())
     suite.addTest(php.suite())
+    suite.addTest(pygments.suite())
     return suite
 
 if __name__ == '__main__':
