@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005 Edgewall Software
-# Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
+# Copyright (C) 2005-2008 Edgewall Software
+# Copyright (C) 2005-2007 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -11,11 +11,6 @@
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
-#
-# Author: Christopher Lenz <cmlenz@gmx.de>
-
-from trac.config import *
-from trac.test import TestConfiguration
 
 import os
 from StringIO import StringIO
@@ -23,6 +18,9 @@ import tempfile
 import time
 import unittest
 
+from trac.config import *
+from trac.test import Configuration
+
 
 class ConfigurationTestCase(unittest.TestCase):
 
@@ -37,7 +35,7 @@ class ConfigurationTestCase(unittest.TestCase):
         os.remove(self.filename)
 
     def _read(self):
-        return TestConfiguration(self.filename)
+        return Configuration(self.filename)
 
     def _write(self, lines):
         fileobj = open(self.filename, 'w')
@@ -70,7 +68,7 @@ class ConfigurationTestCase(unittest.TestCase):
     def test_default_int(self):
         config = self._read()
         self.assertRaises(ConfigurationError, config.getint, 'a', 'option', 'b')
-        self.assertEquals(None, config.getint('a', 'option'))
+        self.assertEquals(0, config.getint('a', 'option'))
         self.assertEquals(1, config.getint('a', 'option', '1'))
         self.assertEquals(1, config.getint('a', 'option', 1))
 
@@ -84,24 +82,37 @@ class ConfigurationTestCase(unittest.TestCase):
         config = self._read()
         self.assertEquals('x', config.get('a', 'option'))
         self.assertEquals('x', config.get('a', 'option', 'y'))
+        self.assertEquals('y', config.get('b', 'option2', 'y'))
 
     def test_read_and_getbool(self):
         self._write(['[a]', 'option = yes'])
         config = self._read()
         self.assertEquals(True, config.getbool('a', 'option'))
         self.assertEquals(True, config.getbool('a', 'option', False))
+        self.assertEquals(False, config.getbool('b', 'option2'))
+        self.assertEquals(False, config.getbool('b', 'option2', False))
+        self.assertEquals(False, config.getbool('b', 'option2', 'disabled'))
 
     def test_read_and_getint(self):
         self._write(['[a]', 'option = 42'])
         config = self._read()
         self.assertEquals(42, config.getint('a', 'option'))
         self.assertEquals(42, config.getint('a', 'option', 25))
+        self.assertEquals(0, config.getint('b', 'option2'))
+        self.assertEquals(25, config.getint('b', 'option2', 25))
+        self.assertEquals(25, config.getint('b', 'option2', '25'))
 
     def test_read_and_getlist(self):
         self._write(['[a]', 'option = foo, bar, baz'])
         config = self._read()
         self.assertEquals(['foo', 'bar', 'baz'],
                           config.getlist('a', 'option'))
+        self.assertEquals([],
+                          config.getlist('b', 'option2'))
+        self.assertEquals(['foo', 'bar', 'baz'],
+                    config.getlist('b', 'option2', ['foo', 'bar', 'baz']))
+        self.assertEquals(['foo', 'bar', 'baz'],
+                    config.getlist('b', 'option2', 'foo, bar, baz'))
 
     def test_read_and_getlist_sep(self):
         self._write(['[a]', 'option = foo | bar | baz'])
@@ -136,7 +147,7 @@ class ConfigurationTestCase(unittest.TestCase):
                            'option0 = x\n', 
                            "option1 = Voilà l'été\n", 
                            "option2 = Voilà l'été\n", 
-                           # "option3 = Voilà l'été\n", 
+                           # "option3 = VoilÃ  l'Ã©tÃ©\n", 
                            '\n',
                            '[b]\n',
                            'option0 = y\n', 
@@ -149,12 +160,38 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(u"Voilà l'été", config2.get('a', 'option2'))
         # self.assertEquals(u"Voilà l'été", config2.get('a', 'option3'))
 
+    def test_set_and_save_inherit(self):
+        def testcb():
+            config = self._read()
+            config.set('a', 'option2', "Voilà l'été")  # UTF-8
+            config.set('a', 'option1', u"Voilà l'été") # unicode
+            self.assertEquals('x', config.get('a', 'option'))
+            self.assertEquals(u"Voilà l'été", config.get('a', 'option1'))
+            self.assertEquals(u"Voilà l'été", config.get('a', 'option2'))
+            config.save()
+
+            configfile = open(self.filename, 'r')
+            self.assertEquals(['# -*- coding: utf-8 -*-\n',
+                               '\n',
+                               '[a]\n',
+                               "option1 = Voilà l'été\n", 
+                               "option2 = Voilà l'été\n", 
+                               '\n',
+                               '[inherit]\n',
+                               "file = trac-site.ini\n", 
+                               '\n'],
+                              configfile.readlines())
+            configfile.close()
+            config2 = Configuration(self.filename)
+            self.assertEquals('x', config2.get('a', 'option'))
+            self.assertEquals(u"Voilà l'été", config2.get('a', 'option1'))
+            self.assertEquals(u"Voilà l'été", config2.get('a', 'option2'))
+        self._test_with_inherit(testcb)
+
     def test_sections(self):
         self._write(['[a]', 'option = x', '[b]', 'option = y'])
         config = self._read()
         self.assertEquals(['a', 'b'], config.sections())
-        config.site_parser.add_section('c')
-        self.assertEquals(['a', 'b', 'c'], config.sections())
 
     def test_options(self):
         self._write(['[a]', 'option = x', '[b]', 'option = y'])
@@ -163,16 +200,50 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertEquals(('option', 'y'), iter(config.options('b')).next())
         self.assertRaises(StopIteration, iter(config.options('c')).next)
 
+    def test_has_option(self):
+        config = self._read()
+        self.assertEquals(False, config.has_option('a', 'option'))
+        self._write(['[a]', 'option = x'])
+        config = self._read()
+        self.assertEquals(True, config.has_option('a', 'option'))
+
     def test_reparse(self):
         self._write(['[a]', 'option = x'])
         config = self._read()
         self.assertEquals('x', config.get('a', 'option'))
-        time.sleep(1) # needed because of low mtime granularity
+        time.sleep(2) # needed because of low mtime granularity,
+                      # especially on fat filesystems
 
         self._write(['[a]', 'option = y'])
         config.parse_if_needed()
         self.assertEquals('y', config.get('a', 'option'))
 
+    def test_inherit_one_level(self):
+        def testcb():
+            config = self._read()
+            self.assertEqual('x', config.get('a', 'option'))
+            self.assertEqual(['a', 'inherit'], config.sections())
+            config.remove('a', 'option') # Should *not* remove option in parent
+            self.assertEqual('x', config.get('a', 'option'))
+            self.assertEqual([('option', 'x')], list(config.options('a')))
+            self.assertEqual(True, 'a' in config)
+        self._test_with_inherit(testcb)
+
+
+    def _test_with_inherit(self, testcb):
+        sitename = os.path.join(tempfile.gettempdir(), 'trac-site.ini')
+        sitefile = open(sitename, 'w')
+        try:
+            try:
+                sitefile.write('[a]\noption = x\n')
+            finally:
+                sitefile.close()
+
+            self._write(['[inherit]', 'file = trac-site.ini'])
+            testcb()
+        finally:
+            os.remove(sitename)
+
 
 def suite():
     return unittest.makeSuite(ConfigurationTestCase, 'test')
