@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2006 Edgewall Software
+# Copyright (C) 2003-2008 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
@@ -20,11 +20,15 @@
 import re
 import sys
 import time
+from datetime import date, datetime
 
 from trac.attachment import Attachment
 from trac.core import TracError
-from trac.ticket import TicketSystem
+from trac.resource import Resource, ResourceNotFound
+from trac.ticket.api import TicketSystem
 from trac.util import sorted, embedded_numbers
+from trac.util.datefmt import utc, utcmax, to_timestamp
+from trac.util.translation import _
 
 __all__ = ['Ticket', 'Type', 'Status', 'Resolution', 'Priority', 'Severity',
            'Component', 'Milestone', 'Version']
@@ -32,8 +36,11 @@ __all__ = ['Ticket', 'Type', 'Status', 'Resolution', 'Priority', 'Severity',
 
 class Ticket(object):
 
-    def __init__(self, env, tkt_id=None, db=None):
+    id_is_valid = staticmethod(lambda num: 0 < int(num) <= 1L << 31)
+
+    def __init__(self, env, tkt_id=None, db=None, version=None):
         self.env = env
+        self.resource = Resource('ticket', tkt_id, version)
         self.fields = TicketSystem(self.env).get_ticket_fields()
         self.values = {}
         if tkt_id is not None:
@@ -57,7 +64,10 @@ class Ticket(object):
     def _init_defaults(self, db=None):
         for field in self.fields:
             default = None
-            if not field.get('custom'):
+            if field['name'] in ['resolution', 'status']:
+                # Ignore for new - only change through workflow
+                pass
+            elif not field.get('custom'):
                 default = self.env.config.get('ticket',
                                               'default_' + field['name'])
             else:
@@ -74,23 +84,25 @@ class Ticket(object):
                 self.values.setdefault(field['name'], default)
 
     def _fetch_ticket(self, tkt_id, db=None):
-        db = self._get_db(db)
+        row = None
+        if self.id_is_valid(tkt_id):
+            db = self._get_db(db)
 
-        # Fetch the standard ticket fields
-        std_fields = [f['name'] for f in self.fields if not f.get('custom')]
-        cursor = db.cursor()
-        cursor.execute("SELECT %s,time,changetime FROM ticket WHERE id=%%s"
-                       % ','.join(std_fields), (tkt_id,))
-        row = cursor.fetchone()
+            # Fetch the standard ticket fields
+            std_fields = [f['name'] for f in self.fields if not f.get('custom')]
+            cursor = db.cursor()
+            cursor.execute("SELECT %s,time,changetime FROM ticket WHERE id=%%s"
+                           % ','.join(std_fields), (tkt_id,))
+            row = cursor.fetchone()
         if not row:
-            raise TracError('Ticket %d does not exist.' % tkt_id,
-                            'Invalid Ticket Number')
+            raise ResourceNotFound('Ticket %s does not exist.' % tkt_id,
+                                   'Invalid Ticket Number')
 
         self.id = tkt_id
         for i in range(len(std_fields)):
             self.values[std_fields[i]] = row[i] or ''
-        self.time_created = row[len(std_fields)]
-        self.time_changed = row[len(std_fields) + 1]
+        self.time_created = datetime.fromtimestamp(row[len(std_fields)], utc)
+        self.time_changed = datetime.fromtimestamp(row[len(std_fields) + 1], utc)
 
         # Fetch custom fields if available
         custom_fields = [f['name'] for f in self.fields if f.get('custom')]
@@ -101,17 +113,19 @@ class Ticket(object):
                 self.values[name] = value
 
     def __getitem__(self, name):
-        return self.values[name]
+        return self.values.get(name)
 
     def __setitem__(self, name, value):
         """Log ticket modifications so the table ticket_change can be updated"""
-        if self.values.has_key(name) and self.values[name] == value:
+        if name in self.values and self.values[name] == value:
             return
-        if not self._old.has_key(name): # Changed field
+        if name not in self._old: # Changed field
             self._old[name] = self.values.get(name)
         elif self._old[name] == value: # Change of field reverted
             del self._old[name]
         if value:
+            if isinstance(value, list):
+                raise TracError(_("Multi-values fields not supported yet"))
             field = [field for field in self.fields if field['name'] == name]
             if field and field[0].get('type') != 'textarea':
                 value = value.strip()
@@ -126,18 +140,17 @@ class Ticket(object):
         # We have to do an extra trick to catch unchecked checkboxes
         for name in [name for name in values.keys() if name[9:] in field_names
                      and name.startswith('checkbox_')]:
-            if not values.has_key(name[9:]):
+            if name[9:] not in values:
                 self[name[9:]] = '0'
 
-    def insert(self, when=0, db=None):
+    def insert(self, when=None, db=None):
         """Add ticket to database"""
         assert not self.exists, 'Cannot insert an existing ticket'
         db, handle_ta = self._get_db_for_write(db)
 
         # Add a timestamp
-        if not when:
-            when = time.time()
-        when = int(when)
+        if when is None:
+            when = datetime.now(utc)
         self.time_created = self.time_changed = when
 
         cursor = db.cursor()
@@ -148,28 +161,33 @@ class Ticket(object):
                 component = Component(self.env, self['component'], db=db)
                 if component.owner:
                     self['owner'] = component.owner
-            except TracError, e:
-                # Assume that no such component exists
+            except ResourceNotFound, e:
+                # No such component exists
                 pass
 
         # Insert ticket record
-        std_fields = [f['name'] for f in self.fields if not f.get('custom')
-                      and self.values.has_key(f['name'])]
+        created = to_timestamp(self.time_created)
+        changed = to_timestamp(self.time_changed)
+        std_fields = []
+        custom_fields = []
+        for f in self.fields:
+            fname = f['name']
+            if fname in self.values:
+                if f.get('custom'):
+                    custom_fields.append(fname)
+                else:
+                    std_fields.append(fname)
         cursor.execute("INSERT INTO ticket (%s,time,changetime) VALUES (%s)"
                        % (','.join(std_fields),
                           ','.join(['%s'] * (len(std_fields) + 2))),
-                       [self[name] for name in std_fields] +
-                       [self.time_created, self.time_changed])
+                       [self[name] for name in std_fields] + [created, changed])
         tkt_id = db.get_last_id(cursor, 'ticket')
 
         # Insert custom fields
-        custom_fields = [f['name'] for f in self.fields if f.get('custom')
-                         and self.values.has_key(f['name'])]
         if custom_fields:
             cursor.executemany("INSERT INTO ticket_custom (ticket,name,value) "
                                "VALUES (%s,%s,%s)", [(tkt_id, name, self[name])
                                                      for name in custom_fields])
-
         if handle_ta:
             db.commit()
 
@@ -181,7 +199,7 @@ class Ticket(object):
 
         return self.id
 
-    def save_changes(self, author, comment, when=0, db=None, cnum=''):
+    def save_changes(self, author, comment, when=None, db=None, cnum=''):
         """
         Store ticket changes in the database. The ticket must already exist in
         the database.  Returns False if there were no changes to save, True
@@ -194,14 +212,16 @@ class Ticket(object):
 
         db, handle_ta = self._get_db_for_write(db)
         cursor = db.cursor()
-        when = int(when or time.time())
+        if when is None:
+            when = datetime.now(utc)
+        when_ts = to_timestamp(when)
 
-        if self.values.has_key('component'):
+        if 'component' in self.values:
             # If the component is changed on a 'new' ticket then owner field
             # is updated accordingly. (#623).
             if self.values.get('status') == 'new' \
-                    and self._old.has_key('component') \
-                    and not self._old.has_key('owner'):
+                    and 'component' in self._old \
+                    and 'owner' not in self._old:
                 try:
                     old_comp = Component(self.env, self._old['component'], db)
                     old_owner = old_comp.owner or ''
@@ -215,7 +235,7 @@ class Ticket(object):
                     pass
 
         # Fix up cc list separators and remove duplicates
-        if self.values.has_key('cc'):
+        if 'cc' in self.values:
             cclist = []
             for cc in re.split(r'[;,\s]+', self.values['cc']):
                 if cc not in cclist:
@@ -241,16 +261,16 @@ class Ticket(object):
             cursor.execute("INSERT INTO ticket_change "
                            "(ticket,time,author,field,oldvalue,newvalue) "
                            "VALUES (%s, %s, %s, %s, %s, %s)",
-                           (self.id, when, author, name, self._old[name],
+                           (self.id, when_ts, author, name, self._old[name],
                             self[name]))
         # always save comment, even if empty (numbering support for timeline)
         cursor.execute("INSERT INTO ticket_change "
                        "(ticket,time,author,field,oldvalue,newvalue) "
                        "VALUES (%s,%s,%s,'comment',%s,%s)",
-                       (self.id, when, author, cnum, comment))
+                       (self.id, when_ts, author, cnum, comment))
 
         cursor.execute("UPDATE ticket SET changetime=%s WHERE id=%s",
-                       (when, self.id))
+                       (when_ts, self.id))
 
         if handle_ta:
             db.commit()
@@ -262,7 +282,7 @@ class Ticket(object):
             listener.ticket_changed(self, comment, author, old_values)
         return True
 
-    def get_changelog(self, when=0, db=None):
+    def get_changelog(self, when=None, db=None):
         """Return the changelog as a list of tuples of the form
         (time, author, field, oldvalue, newvalue, permanent).
 
@@ -270,10 +290,10 @@ class Ticket(object):
         the `permanent` flag is used to distinguish collateral changes
         that are not yet immutable (like attachments, currently).
         """
-        when = int(when)
         db = self._get_db(db)
         cursor = db.cursor()
-        if when:
+        when_ts = when and to_timestamp(when) or 0
+        if when_ts:
             cursor.execute("SELECT time,author,field,oldvalue,newvalue,1 "
                            "FROM ticket_change WHERE ticket=%s AND time=%s "
                            "UNION "
@@ -283,7 +303,8 @@ class Ticket(object):
                            "SELECT time,author,'comment',null,description,0 "
                            "FROM attachment WHERE id=%s AND time=%s "
                            "ORDER BY time",
-                           (self.id, when, str(self.id), when, str(self.id), when))
+                           (self.id, when_ts, str(self.id), when_ts, 
+                           str(self.id), when_ts))
         else:
             cursor.execute("SELECT time,author,field,oldvalue,newvalue,1 "
                            "FROM ticket_change WHERE ticket=%s "
@@ -293,11 +314,12 @@ class Ticket(object):
                            "UNION "
                            "SELECT time,author,'comment',null,description,0 "
                            "FROM attachment WHERE id=%s "
-                           "ORDER BY time", (self.id,  str(self.id), str(self.id)))
+                           "ORDER BY time", (self.id,  str(self.id), 
+                           str(self.id)))
         log = []
         for t, author, field, oldvalue, newvalue, permanent in cursor:
-            log.append((int(t), author, field, oldvalue or '', newvalue or '',
-                        permanent))
+            log.append((datetime.fromtimestamp(int(t), utc), author, field,
+                       oldvalue or '', newvalue or '', permanent))
         return log
 
     def delete(self, db=None):
@@ -315,6 +337,11 @@ class Ticket(object):
             listener.ticket_deleted(self)
 
 
+def simplify_whitespace(name):
+    """Strip spaces and remove duplicate spaces within names"""
+    return ' '.join(name.split())
+        
+
 class AbstractEnum(object):
     type = None
     ticket_col = None
@@ -323,6 +350,8 @@ class AbstractEnum(object):
         if not self.ticket_col:
             self.ticket_col = self.type
         self.env = env
+        if name:
+            name = simplify_whitespace(name)
         if name:
             if not db:
                 db = self.env.get_db_cnx()
@@ -331,7 +360,8 @@ class AbstractEnum(object):
                            (self.type, name))
             row = cursor.fetchone()
             if not row:
-                raise TracError, '%s %s does not exist.' % (self.type, name)
+                raise ResourceNotFound(_('%(type)s %(name)s does not exist.',
+                                  type=self.type, name=name))
             self.value = self._old_value = row[0]
             self.name = self._old_name = name
         else:
@@ -341,7 +371,7 @@ class AbstractEnum(object):
     exists = property(fget=lambda self: self._old_value is not None)
 
     def delete(self, db=None):
-        assert self.exists, 'Cannot deleting non-existent %s' % self.type
+        assert self.exists, 'Cannot delete non-existent %s' % self.type
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -352,6 +382,14 @@ class AbstractEnum(object):
         self.env.log.info('Deleting %s %s' % (self.type, self.name))
         cursor.execute("DELETE FROM enum WHERE type=%s AND value=%s",
                        (self.type, self._old_value))
+        # Re-order any enums that have higher value than deleted (close gap)
+        for enum in list(self.select(self.env)):
+            try:
+                if int(enum.value) > int(self._old_value):
+                    enum.value = unicode(int(enum.value) - 1)
+                    enum.update(db=db)
+            except ValueError:
+                pass # Ignore cast error for this non-essential operation
 
         if handle_ta:
             db.commit()
@@ -360,8 +398,8 @@ class AbstractEnum(object):
 
     def insert(self, db=None):
         assert not self.exists, 'Cannot insert existing %s' % self.type
+        self.name = simplify_whitespace(self.name)
         assert self.name, 'Cannot create %s with no name' % self.type
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -385,8 +423,8 @@ class AbstractEnum(object):
 
     def update(self, db=None):
         assert self.exists, 'Cannot update non-existent %s' % self.type
+        self.name = simplify_whitespace(self.name)
         assert self.name, 'Cannot update %s with no name' % self.type
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -429,8 +467,15 @@ class Type(AbstractEnum):
     ticket_col = 'type'
 
 
-class Status(AbstractEnum):
-    type = 'status'
+class Status(object):
+    def __init__(self, env):
+        self.env = env
+    def select(cls, env, db=None):
+        for state in TicketSystem(env).get_all_status():
+            status = cls(env)
+            status.name = state
+            yield status
+    select = classmethod(select)
 
 
 class Resolution(AbstractEnum):
@@ -449,6 +494,8 @@ class Component(object):
 
     def __init__(self, env, name=None, db=None):
         self.env = env
+        if name:
+            name = simplify_whitespace(name)
         if name:
             if not db:
                 db = self.env.get_db_cnx()
@@ -457,7 +504,8 @@ class Component(object):
                            "WHERE name=%s", (name,))
             row = cursor.fetchone()
             if not row:
-                raise TracError, 'Component %s does not exist.' % name
+                raise ResourceNotFound(_('Component %(name)s does not exist.',
+                                  name=name))
             self.name = self._old_name = name
             self.owner = row[0] or None
             self.description = row[1] or ''
@@ -469,7 +517,7 @@ class Component(object):
     exists = property(fget=lambda self: self._old_name is not None)
 
     def delete(self, db=None):
-        assert self.exists, 'Cannot deleting non-existent component'
+        assert self.exists, 'Cannot delete non-existent component'
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -487,8 +535,8 @@ class Component(object):
 
     def insert(self, db=None):
         assert not self.exists, 'Cannot insert existing component'
+        self.name = simplify_whitespace(self.name)
         assert self.name, 'Cannot create component with no name'
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -506,8 +554,8 @@ class Component(object):
 
     def update(self, db=None):
         assert self.exists, 'Cannot update non-existent component'
+        self.name = simplify_whitespace(self.name)
         assert self.name, 'Cannot update component with no name'
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -537,7 +585,7 @@ class Component(object):
                        "ORDER BY name")
         for name, owner, description in cursor:
             component = cls(env)
-            component.name = name
+            component.name = component._old_name = name
             component.owner = owner or None
             component.description = description or ''
             yield component
@@ -553,9 +601,13 @@ class Milestone(object):
             self._old_name = name
         else:
             self.name = self._old_name = None
-            self.due = self.completed = 0
+            self.due = self.completed = None
             self.description = ''
 
+    def _get_resource(self):
+        return Resource('milestone', self.name) ### .version !!!
+    resource = property(_get_resource)
+
     def _fetch(self, name, db=None):
         if not db:
             db = self.env.get_db_cnx()
@@ -564,17 +616,22 @@ class Milestone(object):
                        "FROM milestone WHERE name=%s", (name,))
         row = cursor.fetchone()
         if not row:
-            raise TracError('Milestone %s does not exist.' % name,
-                            'Invalid Milestone Name')
-        self.name = row[0]
-        self.due = row[1] and int(row[1]) or 0
-        self.completed = row[2] and int(row[2]) or 0
-        self.description = row[3] or ''
+            raise ResourceNotFound('Milestone %s does not exist.' % name,
+                                   'Invalid Milestone Name')
+        self._from_database(row)
 
     exists = property(fget=lambda self: self._old_name is not None)
-    is_completed = property(fget=lambda self: self.completed != 0)
+    is_completed = property(fget=lambda self: self.completed is not None)
     is_late = property(fget=lambda self: self.due and \
-                                         self.due < time.time() - 86400)
+                                         self.due.date() < date.today())
+
+    def _from_database(self, row):
+        name, due, completed, description = row
+        self.name = self._old_name = name
+        self.due = due and datetime.fromtimestamp(int(due), utc) or None
+        self.completed = completed and \
+                         datetime.fromtimestamp(int(completed), utc) or None
+        self.description = description or ''
 
     def delete(self, retarget_to=None, author=None, db=None):
         if not db:
@@ -588,7 +645,7 @@ class Milestone(object):
         cursor.execute("DELETE FROM milestone WHERE name=%s", (self.name,))
 
         # Retarget/reset tickets associated with this milestone
-        now = time.time()
+        now = datetime.now(utc)
         cursor.execute("SELECT id FROM ticket WHERE milestone=%s", (self.name,))
         tkt_ids = [int(row[0]) for row in cursor]
         for tkt_id in tkt_ids:
@@ -602,36 +659,38 @@ class Milestone(object):
 
     def insert(self, db=None):
         assert self.name, 'Cannot create milestone with no name'
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
         else:
             handle_ta = False
 
+        self.name = simplify_whitespace(self.name)
         cursor = db.cursor()
         self.env.log.debug("Creating new milestone '%s'" % self.name)
         cursor.execute("INSERT INTO milestone (name,due,completed,description) "
                        "VALUES (%s,%s,%s,%s)",
-                       (self.name, self.due, self.completed, self.description))
+                       (self.name, to_timestamp(self.due), to_timestamp(self.completed),
+                        self.description))
 
         if handle_ta:
             db.commit()
 
     def update(self, db=None):
         assert self.name, 'Cannot update milestone with no name'
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
         else:
             handle_ta = False
 
+        self.name = simplify_whitespace(self.name)
         cursor = db.cursor()
         self.env.log.info('Updating milestone "%s"' % self.name)
         cursor.execute("UPDATE milestone SET name=%s,due=%s,"
                        "completed=%s,description=%s WHERE name=%s",
-                       (self.name, self.due, self.completed, self.description,
+                       (self.name, to_timestamp(self.due), to_timestamp(self.completed),
+                        self.description,
                         self._old_name))
         self.env.log.info('Updating milestone field of all tickets '
                           'associated with milestone "%s"' % self.name)
@@ -651,16 +710,13 @@ class Milestone(object):
         cursor = db.cursor()
         cursor.execute(sql)
         milestones = []
-        for name,due,completed,description in cursor:
+        for row in cursor:
             milestone = Milestone(env)
-            milestone.name = milestone._old_name = name
-            milestone.due = due and int(due) or 0
-            milestone.completed = completed and int(completed) or 0
-            milestone.description = description or ''
+            milestone._from_database(row)
             milestones.append(milestone)
         def milestone_order(m):
-            return (m.completed or sys.maxint,
-                    m.due or sys.maxint,
+            return (m.completed or utcmax,
+                    m.due or utcmax,
                     embedded_numbers(m.name))
         return sorted(milestones, key=milestone_order)
     select = classmethod(select)
@@ -678,9 +734,10 @@ class Version(object):
                            "WHERE name=%s", (name,))
             row = cursor.fetchone()
             if not row:
-                raise TracError, 'Version %s does not exist.' % name
+                raise ResourceNotFound(_('Version %(name)s does not exist.',
+                                  name=name))
             self.name = self._old_name = name
-            self.time = row[0] and int(row[0]) or None
+            self.time = row[0] and datetime.fromtimestamp(int(row[0]), utc) or None
             self.description = row[1] or ''
         else:
             self.name = self._old_name = None
@@ -690,7 +747,7 @@ class Version(object):
     exists = property(fget=lambda self: self._old_name is not None)
 
     def delete(self, db=None):
-        assert self.exists, 'Cannot deleting non-existent version'
+        assert self.exists, 'Cannot delete non-existent version'
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -708,8 +765,8 @@ class Version(object):
 
     def insert(self, db=None):
         assert not self.exists, 'Cannot insert existing version'
+        self.name = simplify_whitespace(self.name)
         assert self.name, 'Cannot create version with no name'
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -720,15 +777,15 @@ class Version(object):
         self.env.log.debug("Creating new version '%s'" % self.name)
         cursor.execute("INSERT INTO version (name,time,description) "
                        "VALUES (%s,%s,%s)",
-                       (self.name, self.time, self.description))
+                       (self.name, to_timestamp(self.time), self.description))
 
         if handle_ta:
             db.commit()
 
     def update(self, db=None):
         assert self.exists, 'Cannot update non-existent version'
+        self.name = simplify_whitespace(self.name)
         assert self.name, 'Cannot update version with no name'
-        self.name = self.name.strip()
         if not db:
             db = self.env.get_db_cnx()
             handle_ta = True
@@ -739,7 +796,7 @@ class Version(object):
         self.env.log.info('Updating version "%s"' % self.name)
         cursor.execute("UPDATE version SET name=%s,time=%s,description=%s "
                        "WHERE name=%s",
-                       (self.name, self.time, self.description,
+                       (self.name, to_timestamp(self.time), self.description,
                         self._old_name))
         if self.name != self._old_name:
             # Update tickets
@@ -758,11 +815,11 @@ class Version(object):
         versions = []
         for name, time, description in cursor:
             version = cls(env)
-            version.name = name
-            version.time = time and int(time) or None
+            version.name = version._old_name = name
+            version.time = time and datetime.fromtimestamp(int(time), utc) or None
             version.description = description or ''
             versions.append(version)
         def version_order(v):
-            return (v.time or sys.maxint, embedded_numbers(v.name))
+            return (v.time or utcmax, embedded_numbers(v.name))
         return sorted(versions, key=version_order, reverse=True)
     select = classmethod(select)
