@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2006 Edgewall Software
+# Copyright (C) 2003-2008 Edgewall Software
 # Copyright (C) 2003-2005 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Emmanuel Blot <emmanuel.blot@free.fr>
 # All rights reserved.
@@ -16,14 +16,15 @@
 # Author: Daniel Lundin <daniel@edgewall.com>
 #
 
-import md5
-
 from trac import __version__
 from trac.core import *
 from trac.config import *
-from trac.util.text import CRLF, wrap
 from trac.notification import NotifyEmail
+from trac.util import md5
+from trac.util.datefmt import to_timestamp
+from trac.util.text import CRLF, wrap, to_unicode
 
+from genshi.template.text import TextTemplate
 
 class TicketNotificationSystem(Component):
 
@@ -40,12 +41,17 @@ class TicketNotificationSystem(Component):
                                        'true',
         """Always send notifications to the person who causes the ticket 
         property change.""")
+        
+    ticket_subject_template = Option('notification', 'ticket_subject_template', 
+                                     '$prefix #$ticket.id: $summary',
+        """A Genshi text template snippet used to get the notification subject.
+        (since 0.11)""")
 
 
 class TicketNotifyEmail(NotifyEmail):
     """Notification of ticket changes."""
 
-    template_name = "ticket_notify_email.cs"
+    template_name = "ticket_notify_email.txt"
     ticket = None
     newticket = None
     modtime = 0
@@ -56,76 +62,89 @@ class TicketNotifyEmail(NotifyEmail):
         NotifyEmail.__init__(self, env)
         self.prev_cc = []
 
-    def notify(self, ticket, newticket=True, modtime=0):
+    def notify(self, ticket, newticket=True, modtime=None):
         self.ticket = ticket
         self.modtime = modtime
         self.newticket = newticket
-        self.ticket['description'] = wrap(self.ticket.values.get('description', ''),
-                                          self.COLS, initial_indent=' ',
-                                          subsequent_indent=' ', linesep=CRLF)
+
+        changes_body = ''
         self.reporter = ''
         self.owner = ''
-        self.hdf.set_unescaped('email.ticket_props', self.format_props())
-        self.hdf.set_unescaped('email.ticket_body_hdr', self.format_hdr())
-        self.hdf['ticket.new'] = self.newticket
-        subject = self.format_subj()
+        changes_descr = ''
+        change_data = {}
         link = self.env.abs_href.ticket(ticket.id)
-        if not self.newticket:
-            subject = 'Re: ' + subject
-        self.hdf.set_unescaped('email.subject', subject)
-        changes = ''
+        summary = self.ticket['summary']
+        
         if not self.newticket and modtime:  # Ticket change
             from trac.ticket.web_ui import TicketModule
             for change in TicketModule(self.env).grouped_changelog_entries(
                 ticket, self.db, when=modtime):
                 if not change['permanent']: # attachment with same time...
                     continue
-                self.hdf.set_unescaped('ticket.change.author', 
-                                       change['author'])
-                self.hdf.set_unescaped('ticket.change.comment',
-                                       wrap(change['comment'], self.COLS,
-                                            ' ', ' ', CRLF))
+                change_data.update({
+                    'author': change['author'],
+                    'comment': wrap(change['comment'], self.COLS, ' ', ' ',
+                                    CRLF)
+                    })
                 link += '#comment:%s' % str(change.get('cnum', ''))
                 for field, values in change['fields'].iteritems():
                     old = values['old']
                     new = values['new']
-                    pfx = 'ticket.change.%s' % field
                     newv = ''
                     if field == 'description':
                         new_descr = wrap(new, self.COLS, ' ', ' ', CRLF)
                         old_descr = wrap(old, self.COLS, '> ', '> ', CRLF)
-                        old_descr = old_descr.replace(2*CRLF, CRLF + '>' + CRLF)
+                        old_descr = old_descr.replace(2*CRLF, CRLF + '>' + \
+                                                      CRLF)
                         cdescr = CRLF
-                        cdescr += 'Old description:' + 2*CRLF + old_descr + 2*CRLF
-                        cdescr += 'New description:' + 2*CRLF + new_descr + CRLF
-                        self.hdf.set_unescaped('email.changes_descr', cdescr)
+                        cdescr += 'Old description:' + 2*CRLF + old_descr + \
+                                  2*CRLF
+                        cdescr += 'New description:' + 2*CRLF + new_descr + \
+                                  CRLF
+                        changes_descr = cdescr
+                    elif field == 'summary':
+                        summary = "%s (was: %s)" % (new, old)
                     elif field == 'cc':
                         (addcc, delcc) = self.diff_cc(old, new)
                         chgcc = ''
                         if delcc:
-                            chgcc += wrap(" * cc: %s (removed)" % ', '.join(delcc), 
-                                          self.COLS, ' ', ' ', CRLF)
-                            chgcc += CRLF
+                            chgcc += wrap(" * cc: %s (removed)" %
+                                          ', '.join(delcc), 
+                                          self.COLS, ' ', ' ', CRLF) + CRLF
                         if addcc:
-                            chgcc += wrap(" * cc: %s (added)" % ', '.join(addcc), 
-                                          self.COLS, ' ', ' ', CRLF)
-                            chgcc += CRLF
+                            chgcc += wrap(" * cc: %s (added)" %
+                                          ', '.join(addcc), 
+                                          self.COLS, ' ', ' ', CRLF) + CRLF
                         if chgcc:
-                            changes += chgcc
+                            changes_body += chgcc
                         self.prev_cc += old and self.parse_cc(old) or []
                     else:
                         newv = new
                         l = 7 + len(field)
                         chg = wrap('%s => %s' % (old, new), self.COLS - l, '',
                                    l * ' ', CRLF)
-                        changes += '  * %s:  %s%s' % (field, chg, CRLF)
+                        changes_body += '  * %s:  %s%s' % (field, chg, CRLF)
                     if newv:
-                        self.hdf.set_unescaped('%s.oldvalue' % pfx, old)
-                        self.hdf.set_unescaped('%s.newvalue' % pfx, newv)
-            if changes:
-                self.hdf.set_unescaped('email.changes_body', changes)
+                        change_data[field] = {'oldvalue': old, 'newvalue': new}
+            
+        self.ticket['description'] = wrap(
+            self.ticket.values.get('description', ''), self.COLS,
+            initial_indent=' ', subsequent_indent=' ', linesep=CRLF)
+        self.ticket['new'] = self.newticket
         self.ticket['link'] = link
-        self.hdf.set_unescaped('ticket', self.ticket.values)
+        
+        subject = self.format_subj(summary)
+        if not self.newticket:
+            subject = 'Re: ' + subject
+        self.data.update({
+            'ticket_props': self.format_props(),
+            'ticket_body_hdr': self.format_hdr(),
+            'subject': subject,
+            'ticket': ticket.values,
+            'changes_body': changes_body,
+            'changes_descr': changes_descr,
+            'change': change_data
+            })
         NotifyEmail.notify(self, ticket.id, subject)
 
     def format_props(self):
@@ -185,15 +204,22 @@ class TicketNotifyEmail(NotifyEmail):
         return '#%s: %s' % (self.ticket.id, wrap(self.ticket['summary'],
                                                  self.COLS, linesep=CRLF))
 
-    def format_subj(self):
+    def format_subj(self, summary):
+        template = self.config.get('notification','ticket_subject_template')
+        template = TextTemplate(template.encode('utf8'))
+                                                
         prefix = self.config.get('notification', 'smtp_subject_prefix')
         if prefix == '__default__': 
-            prefix = '[%s]' % self.config.get('project', 'name') 
-        if prefix: 
-            return '%s #%s: %s' % (prefix, self.ticket.id,
-                                  self.ticket['summary'])
-        else:
-            return '#%s: %s' % (self.ticket.id, self.ticket['summary']) 
+            prefix = '[%s]' % self.config.get('project', 'name')
+        
+        data = {
+            'prefix': prefix,
+            'summary': summary,
+            'ticket': self.ticket,
+            'env': self.env,
+        }
+        
+        return template.generate(**data).render('text', encoding=None).strip()
 
     def get_recipients(self, tktid):
         notify_reporter = self.config.getbool('notification',
@@ -221,7 +247,7 @@ class TicketNotifyEmail(NotifyEmail):
                 torecipients.append(row[2])
 
         # Harvest email addresses from the author field of ticket_change(s)
-        if notify_reporter:
+        if notify_updater:
             cursor.execute("SELECT DISTINCT author,ticket FROM ticket_change "
                            "WHERE ticket=%s", (tktid,))
             for author,ticket in cursor:
@@ -252,12 +278,12 @@ class TicketNotifyEmail(NotifyEmail):
 
         return (torecipients, ccrecipients)
 
-    def get_message_id(self, rcpt, modtime=0):
+    def get_message_id(self, rcpt, modtime=None):
         """Generate a predictable, but sufficiently unique message ID."""
         s = '%s.%08d.%d.%s' % (self.config.get('project', 'url'),
-                               int(self.ticket.id), modtime, 
+                               int(self.ticket.id), to_timestamp(modtime),
                                rcpt.encode('ascii', 'ignore'))
-        dig = md5.new(s).hexdigest()
+        dig = md5(s).hexdigest()
         host = self.from_email[self.from_email.find('@') + 1:]
         msgid = '<%03d.%s@%s>' % (len(s), dig, host)
         return msgid
