@@ -14,7 +14,8 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from heapq import heappop, heappush
+import os.path
+
 try:
     import threading
 except ImportError:
@@ -24,12 +25,17 @@ except ImportError:
 from trac.config import Option
 from trac.core import *
 from trac.perm import PermissionError
+from trac.resource import IResourceManager, ResourceSystem, ResourceNotFound
+from trac.util.text import to_unicode
+from trac.util.translation import _
 from trac.web.api import IRequestFilter
 
 
 class IRepositoryConnector(Interface):
     """Provide support for a specific version control system."""
 
+    error = None # place holder for storing relevant error message
+
     def get_supported_types():
         """Return the types of version control systems that are supported.
 
@@ -38,6 +44,11 @@ class IRepositoryConnector(Interface):
         
         If multiple provider match a given type, the `priority` is used to
         choose between them (highest number is highest priority).
+
+        If the `priority` returned is negative, this indicates that the 
+        connector for the given `repotype` indeed exists but can't be
+        used for some reason. The `error` property can then be used to 
+        store an error message or exception relevant to the problem detected.
         """
 
     def get_repository(repos_type, repos_dir, authname):
@@ -51,14 +62,15 @@ class RepositoryManager(Component):
     It provides easy access to the configured implementation.
     """
 
-    implements(IRequestFilter)
+    implements(IRequestFilter, IResourceManager)
 
     connectors = ExtensionPoint(IRepositoryConnector)
 
     repository_type = Option('trac', 'repository_type', 'svn',
         """Repository connector type. (''since 0.10'')""")
     repository_dir = Option('trac', 'repository_dir', '',
-        """Path to local repository""")
+        """Path to local repository. This can also be a relative path
+        (''since 0.11'').""")
 
     def __init__(self):
         self._cache = {}
@@ -68,39 +80,80 @@ class RepositoryManager(Component):
     # IRequestFilter methods
 
     def pre_process_request(self, req, handler):
-        from trac.web.chrome import Chrome        
+        from trac.web.chrome import Chrome, add_warning
         if handler is not Chrome(self.env):
-            self.get_repository(req.authname).sync()
+            try:
+                self.get_repository(req.authname).sync()
+            except TracError, e:
+                add_warning(req, _("Can't synchronize with the repository "
+                              "(%(error)s). Look in the Trac log for more "
+                              "information.", error=e.message))
+                          
         return handler
 
     def post_process_request(self, req, template, content_type):
         return (template, content_type)
 
+    # IResourceManager methods
+
+    def get_resource_realms(self):
+        yield 'changeset'
+        yield 'source'
+
+    def get_resource_description(self, resource, format=None, **kwargs):
+        if resource.realm == 'changeset':
+            return _("Changeset %(rev)s", rev=resource.id)
+        elif resource.realm == 'source':
+            version = ''
+            if format == 'summary':
+                repos = resource.env.get_repository() # no perm.username!
+                node = repos.get_node(resource.id, resource.version)
+                if node.isdir:
+                    kind = _("Directory")
+                elif node.isfile:
+                    kind = _("File")
+                if resource.version:
+                    version = _("at version %(rev)s", rev=resource.version)
+            else:
+                kind = _("Path")
+                if resource.version:
+                    version = '@%s' % resource.version
+            return '%s %s%s' % (kind, resource.id, version)
 
     # Public API methods
 
     def get_repository(self, authname):
-        if not self._connector:
-            candidates = []
-            for connector in self.connectors:
-                for repos_type_, prio in connector.get_supported_types():
-                    if self.repository_type != repos_type_:
-                        continue
-                    heappush(candidates, (-prio, connector))
-            if not candidates:
-                raise TracError('Unsupported version control system "%s". '
-                                'Check that the Python bindings for "%s" are '
-                                'correctly installed.' %
-                                ((self.repository_type,)*2))
-            self._connector = heappop(candidates)[1]
         db = self.env.get_db_cnx() # prevent possible deadlock, see #4465
         try:
             self._lock.acquire()
+            if not self._connector:
+                candidates = [
+                    (prio, connector)
+                    for connector in self.connectors
+                    for repos_type, prio in connector.get_supported_types()
+                    if repos_type == self.repository_type
+                ]
+                if candidates:
+                    prio, connector = max(candidates)
+                    if prio < 0: # error condition
+                        raise TracError(
+                            _('Unsupported version control system "%(name)s"'
+                              ': "%(error)s" ', name=self.repository_type, 
+                              error=connector.error))
+                    self._connector = connector
+                else:
+                    raise TracError(
+                        _('Unsupported version control system "%(name)s": '
+                          'Can\'t find an appropriate component, maybe the '
+                          'corresponding plugin was not enabled? ',
+                          name=self.repository_type))
             tid = threading._get_ident()
             if tid in self._cache:
                 repos = self._cache[tid]
             else:
                 rtype, rdir = self.repository_type, self.repository_dir
+                if not os.path.isabs(rdir):
+                    rdir = os.path.join(self.env.path, rdir)
                 repos = self._connector.get_repository(rtype, rdir, authname)
                 self._cache[tid] = repos
             return repos
@@ -112,19 +165,25 @@ class RepositoryManager(Component):
             assert tid == threading._get_ident()
             try:
                 self._lock.acquire()
-                self._cache.pop(tid, None)
+                repos = self._cache.pop(tid, None)
+                if repos:
+                    repos.close()
             finally:
                 self._lock.release()
 
 
-class NoSuchChangeset(TracError):
+class NoSuchChangeset(ResourceNotFound):
     def __init__(self, rev):
-        TracError.__init__(self, "No changeset %s in the repository" % rev)
+        ResourceNotFound.__init__(self,
+                                  _('No changeset %(rev)s in the repository',
+                                    rev=rev),
+                                  _('No such changeset'))
 
-class NoSuchNode(TracError):
+class NoSuchNode(ResourceNotFound):
     def __init__(self, path, rev, msg=None):
-        TracError.__init__(self, "%sNo node %s at revision %s" \
-                           % (msg and '%s: ' % msg or '', path, rev))
+        ResourceNotFound.__init__(self, "%sNo node %s at revision %s" %
+                                  ((msg and '%s: ' % msg) or '', path, rev),
+                                  _('No such node'))
 
 class Repository(object):
     """Base class for a repository provided by a version control system."""
@@ -160,6 +219,16 @@ class Repository(object):
         """Resync the repository cache for the given `rev`, if relevant."""
         raise NotImplementedError
 
+    def get_quickjump_entries(self, rev):
+        """Generate a list of interesting places in the repository.
+
+        `rev` might be used to restrict the list of available locations,
+        but in general it's best to produce all known locations.
+
+        The generated results must be of the form (category, name, path, rev).
+        """
+        return []
+    
     def get_changeset(self, rev):
         """Retrieve a Changeset corresponding to the  given revision `rev`."""
         raise NotImplementedError
@@ -209,7 +278,7 @@ class Repository(object):
         raise NotImplementedError
     youngest_rev = property(lambda x: x.get_youngest_rev())
 
-    def previous_rev(self, rev):
+    def previous_rev(self, rev, path=''):
         """Return the revision immediately preceding the specified revision."""
         raise NotImplementedError
 
@@ -250,7 +319,7 @@ class Repository(object):
 
     def normalize_path(self, path):
         """Return a canonical representation of path in the repos."""
-        return NotImplementedError
+        raise NotImplementedError
 
     def normalize_rev(self, rev):
         """Return a canonical representation of a revision.
@@ -263,7 +332,7 @@ class Repository(object):
         In addition, if `rev` is `None` or '', the youngest revision should
         be returned.
         """
-        return NotImplementedError
+        raise NotImplementedError
 
     def short_rev(self, rev):
         """Return a compact representation of a revision in the repos."""
@@ -299,7 +368,7 @@ class Node(object):
     def __init__(self, path, rev, kind):
         assert kind in (Node.DIRECTORY, Node.FILE), \
                "Unknown node kind %s" % kind
-        self.path = unicode(path)
+        self.path = to_unicode(path)
         self.rev = rev
         self.kind = kind
 
@@ -343,6 +412,15 @@ class Node(object):
             else:
                 return p
 
+    def get_annotations(self):
+        """Provide detailed backward history for the content of this Node.
+
+        Retrieve an array of revisions, one `rev` for each line of content
+        for that node.
+        Only expected to work on (text) FILE nodes, of course.
+        """
+        raise NotImplementedError
+
     def get_properties(self):
         """Returns the properties (meta-data) of the node, as a dictionary.
 
@@ -389,26 +467,25 @@ class Changeset(object):
 
     # change types which can have diff associated to them
     DIFF_CHANGES = (EDIT, COPY, MOVE) # MERGE
+    OTHER_CHANGES = (ADD, DELETE)
+    ALL_CHANGES = DIFF_CHANGES + OTHER_CHANGES
 
     def __init__(self, rev, message, author, date):
         self.rev = rev
-        self.message = message
-        self.author = author
+        self.message = message or ''
+        self.author = author or ''
         self.date = date
 
     def get_properties(self):
-        """Generator that provide additional metadata for this changeset.
+        """Returns the properties (meta-data) of the node, as a dictionary.
 
-        Each additional property is a 4 element tuple:
-         * `name` is the name of the property,
-         * `text` its value
-         * `wikiflag` indicates whether the `text` should be interpreted as
-            wiki text or not
-         * `htmlclass` enables to attach special formatting to the displayed
-            property, e.g. `'author'`, `'time'`, `'message'` or `'changeset'`.
+        The set of properties depends on the version control system.
 
-        Warning: API will be improved (see #1601 and #2545).
+        Warning: this used to yield 4-elements tuple (besides `name` and
+        `text`, there were `wikiflag` and `htmlclass` values).
+        This is now replaced by the usage of IPropertyRenderer (see #1601).
         """
+        return []
         
     def get_changes(self):
         """Generator that produces a tuple for every change in the changeset
@@ -417,6 +494,10 @@ class Changeset(object):
         where `change` can be one of Changeset.ADD, Changeset.COPY,
         Changeset.DELETE, Changeset.EDIT or Changeset.MOVE,
         and `kind` is one of Node.FILE or Node.DIRECTORY.
+        The `path` is the targeted path for the `change` (which is
+        the ''deleted'' path  for a DELETE change).
+        The `base_path` and `base_rev` are the source path and rev for the
+        action (`None` and `-1` in the case of an ADD change).
         """
         raise NotImplementedError
 
@@ -440,13 +521,13 @@ class Authorizer(object):
 
     def assert_permission(self, path):
         if not self.has_permission(path):
-            raise PermissionDenied, \
-                  'Insufficient permissions to access %s' % path
+            raise PermissionDenied(_('Insufficient permissions to access '
+                                     '%(path)s', path=path))
 
     def assert_permission_for_changeset(self, rev):
         if not self.has_permission_for_changeset(rev):
-            raise PermissionDenied, \
-                  'Insufficient permissions to access changeset %s' % rev
+            raise PermissionDenied(_('Insufficient permissions to access '
+                                     'changeset %(id)s', id=rev))
 
     def has_permission(self, path):
         return True
