@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2005-2006 Edgewall Software
+# Copyright (C) 2005-2008 Edgewall Software
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
@@ -15,8 +15,11 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 import posixpath
+from datetime import datetime
 
 from trac.core import TracError
+from trac.util.datefmt import utc, to_timestamp
+from trac.util.translation import _
 from trac.versioncontrol import Changeset, Node, Repository, Authorizer, \
                                 NoSuchChangeset
 
@@ -34,23 +37,34 @@ CACHE_METADATA_KEYS = (CACHE_REPOSITORY_DIR, CACHE_YOUNGEST_REV)
 
 class CachedRepository(Repository):
 
-    def __init__(self, db, repos, authz, log):
+    has_linear_changesets = False
+
+    def __init__(self, getdb, repos, authz, log):
         Repository.__init__(self, repos.name, authz, log)
-        self.db = db
+        if callable(getdb):
+            self.getdb = getdb
+        else:
+            self.getdb = lambda: getdb
         self.repos = repos
 
     def close(self):
         self.repos.close()
 
+    def get_quickjump_entries(self, rev):
+        for category, name, path, rev in self.repos.get_quickjump_entries(rev):
+            yield category, name, path, rev
+
     def get_changeset(self, rev):
         return CachedChangeset(self.repos, self.repos.normalize_rev(rev),
-                               self.db, self.authz)
+                               self.getdb, self.authz)
 
     def get_changesets(self, start, stop):
-        cursor = self.db.cursor()
+        db = self.getdb()
+        cursor = db.cursor()
         cursor.execute("SELECT rev FROM revision "
                        "WHERE time >= %s AND time < %s "
-                       "ORDER BY time", (start, stop))
+                       "ORDER BY time DESC, rev DESC",
+                       (to_timestamp(start), to_timestamp(stop)))
         for rev, in cursor:
             try:
                 if self.authz.has_permission_for_changeset(rev):
@@ -60,15 +74,17 @@ class CachedRepository(Repository):
 
     def sync_changeset(self, rev):
         cset = self.repos.get_changeset(rev)
-        cursor = self.db.cursor()
+        db = self.getdb()
+        cursor = db.cursor()
         cursor.execute("UPDATE revision SET time=%s, author=%s, message=%s "
-                       "WHERE rev=%s", (cset.date, cset.author, cset.message,
+                       "WHERE rev=%s", (to_timestamp(cset.date),
+                                        cset.author, cset.message,
                                         (str(cset.rev))))
-        self.db.commit()
+        db.commit()
         
     def sync(self, feedback=None):
-        cursor = self.db.cursor()
-
+        db = self.getdb()
+        cursor = db.cursor()
         cursor.execute("SELECT name, value FROM system WHERE name IN (%s)" %
                        ','.join(["'%s'" % key for key in CACHE_METADATA_KEYS]))
         metadata = {}
@@ -81,8 +97,8 @@ class CachedRepository(Repository):
             if repository_dir != self.name:
                 self.log.info("'repository_dir' has changed from %r to %r"
                               % (repository_dir, self.name))
-                raise TracError("The 'repository_dir' has changed, "
-                                "a 'trac-admin resync' operation is needed.")
+                raise TracError(_("The 'repository_dir' has changed, a "
+                                  "'trac-admin resync' operation is needed."))
         elif repository_dir is None: # 
             self.log.info('Storing initial "repository_dir": %s' % self.name)
             cursor.execute("INSERT INTO system (name,value) VALUES (%s,%s)",
@@ -92,7 +108,7 @@ class CachedRepository(Repository):
             cursor.execute("UPDATE system SET value=%s WHERE name=%s",
                            (self.name, CACHE_REPOSITORY_DIR))
 
-        self.db.commit() # save metadata changes made up to now
+        db.commit() # save metadata changes made up to now
 
         # -- retrieve the youngest revision in the repository
         self.repos.clear()
@@ -100,7 +116,7 @@ class CachedRepository(Repository):
 
         # -- retrieve the youngest revision cached so far
         if CACHE_YOUNGEST_REV not in metadata:
-            raise TracError('Missing "youngest_rev" in cache metadata')
+            raise TracError(_('Missing "youngest_rev" in cache metadata'))
         
         self.youngest = metadata[CACHE_YOUNGEST_REV]
 
@@ -124,6 +140,12 @@ class CachedRepository(Repository):
                 next_youngest = None
                 try:
                     next_youngest = self.repos.oldest_rev
+                    # Ugly hack needed because doing that everytime in 
+                    # oldest_rev suffers from horrendeous performance (#5213)
+                    if hasattr(self.repos, 'scope'):
+                        if self.repos.scope != '/':
+                            next_youngest = self.repos.next_rev(next_youngest, 
+                                    find_initial_rev=True)
                     next_youngest = self.repos.normalize_rev(next_youngest)
                 except TracError:
                     return # can't normalize oldest_rev: repository was empty
@@ -160,7 +182,8 @@ class CachedRepository(Repository):
                         cursor.execute("INSERT INTO revision "
                                        " (rev,time,author,message) "
                                        "VALUES (%s,%s,%s,%s)",
-                                       (str(next_youngest), cset.date,
+                                       (str(next_youngest),
+                                        to_timestamp(cset.date),
                                         cset.author, cset.message))
                     except Exception, e: # *another* 1.1. resync attempt won 
                         self.log.warning('Revision %s already cached: %s' %
@@ -168,7 +191,7 @@ class CachedRepository(Repository):
                         # also potentially in progress, so keep ''previous''
                         # notion of 'youngest'
                         self.repos.clear(youngest_rev=self.youngest)
-                        self.db.rollback()
+                        db.rollback()
                         return
 
                     # 1.2. now *only* one process was able to get there
@@ -191,10 +214,11 @@ class CachedRepository(Repository):
                     self.youngest = next_youngest                    
                     next_youngest = self.repos.next_rev(next_youngest)
 
-                    # 1.4. update 'youngest_rev' metadata (minimize failures at 0.)
+                    # 1.4. update 'youngest_rev' metadata 
+                    #      (minimize possibility of failures at point 0.)
                     cursor.execute("UPDATE system SET value=%s WHERE name=%s",
                                    (str(self.youngest), CACHE_YOUNGEST_REV))
-                    self.db.commit()
+                    db.commit()
 
                     # 1.5. provide some feedback
                     if feedback:
@@ -217,11 +241,51 @@ class CachedRepository(Repository):
             self.sync()
         return self.youngest
 
-    def previous_rev(self, rev):
-        return self.repos.previous_rev(rev)
+    def previous_rev(self, rev, path=''):
+        if self.has_linear_changesets:
+            return self._next_prev_rev('<', rev, path)
+        else:
+            return self.repos.previous_rev(rev, path)
 
     def next_rev(self, rev, path=''):
-        return self.repos.next_rev(rev, path)
+        if self.has_linear_changesets:
+            return self._next_prev_rev('>', rev, path)
+        else:
+            return self.repos.next_rev(rev, path)
+
+    def _next_prev_rev(self, direction, rev, path=''):
+        db = self.getdb()
+        # the changeset revs are sequence of ints:
+        sql = "SELECT rev FROM node_change WHERE " + \
+              db.cast('rev', 'int') + " " + direction + " %s"
+        args = [rev]
+
+        if path:
+            path = path.lstrip('/')
+            sql += " AND ("
+            # changes on path itself
+            sql += "path=%s "
+            args.append(path)
+            sql += " OR "
+            # changes on path children
+            sql += "path "+db.like()
+            args.append(db.like_escape(path+'/') + '%')
+            sql += " OR "
+            # deletion of path ancestors
+            components = path.lstrip('/').split('/')
+            for i in range(1, len(components)+1):
+                args.append('/'.join(components[:i]))
+            parent_insert = ','.join(('%s',) * len(components))
+            sql += " (path in (" + parent_insert + ") and change_type='D')"
+            sql += ")"
+
+        sql += " ORDER BY " + db.cast('rev', 'int') + \
+                (direction == '<' and " DESC" or "") + " LIMIT 1"
+        
+        cursor = db.cursor()
+        cursor.execute(sql, args)
+        for rev, in cursor:
+            return rev
 
     def rev_older_than(self, rev1, rev2):
         return self.repos.rev_older_than(rev1, rev2)
@@ -235,28 +299,34 @@ class CachedRepository(Repository):
     def normalize_rev(self, rev):
         return self.repos.normalize_rev(rev)
 
-    def get_changes(self, old_path, old_rev, new_path, new_rev, ignore_ancestry=1):
-        return self.repos.get_changes(old_path, old_rev, new_path, new_rev, ignore_ancestry)
+    def get_changes(self, old_path, old_rev, new_path, new_rev, 
+            ignore_ancestry=1):
+        return self.repos.get_changes(old_path, old_rev, new_path, new_rev, 
+                ignore_ancestry)
 
 
 class CachedChangeset(Changeset):
 
-    def __init__(self, repos, rev, db, authz):
-        self.db = db
+    def __init__(self, repos, rev, getdb, authz):
+        self.repos = repos
+        self.getdb = getdb
         self.authz = authz
-        cursor = self.db.cursor()
+        db = self.getdb()
+        cursor = db.cursor()
         cursor.execute("SELECT time,author,message FROM revision "
                        "WHERE rev=%s", (str(rev),))
         row = cursor.fetchone()
         if row:
-            date, author, message = row
-            Changeset.__init__(self, rev, message, author, int(date))
+            _date, author, message = row
+            date = datetime.fromtimestamp(_date, utc)
+            Changeset.__init__(self, rev, message, author, date)
         else:
             raise NoSuchChangeset(rev)
         self.scope = getattr(repos, 'scope', '')
 
     def get_changes(self):
-        cursor = self.db.cursor()
+        db = self.getdb()
+        cursor = db.cursor()
         cursor.execute("SELECT path,node_type,change_type,base_path,base_rev "
                        "FROM node_change WHERE rev=%s "
                        "ORDER BY path", (str(self.rev),))
@@ -270,4 +340,4 @@ class CachedChangeset(Changeset):
             yield path, kind, change, base_path, base_rev
 
     def get_properties(self):
-        return []
+        return self.repos.get_changeset(self.rev).get_properties()
