@@ -1,27 +1,26 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2006 Edgewall Software
+# Copyright (C) 2004-2008 Edgewall Software
 # Copyright (C) 2004-2006 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
-
-from trac.util import enum, escape, Markup
+from trac.util.html import escape, Markup
+from trac.util.text import expandtabs
 
 from difflib import SequenceMatcher
 import re
 
-__all__ = ['get_diff_options', 'hdf_diff', 'unified_diff']
+__all__ = ['get_diff_options', 'hdf_diff', 'diff_blocks', 'unified_diff']
 
 
 def _get_change_extent(str1, str2):
@@ -106,14 +105,14 @@ def _group_opcodes(opcodes, n=3):
     """
     # Full context produces all the opcodes
     if n is None:
-        yield opcodes
+        yield list(opcodes)
         return
 
     # Otherwise we leave at most n lines with the tag 'equal' before and after
     # every change
     nn = n + n
     group = []
-    for idx, (tag, i1, i2, j1, j2) in enum(opcodes):
+    for idx, (tag, i1, i2, j1, j2) in enumerate(opcodes):
         if idx == 0 and tag == 'equal': # Fixup leading unchanged block
             i1, j1 = max(i1, i2 - n), max(j1, j2 - n)
         elif tag == 'equal' and i2 - i1 > nn:
@@ -129,11 +128,14 @@ def _group_opcodes(opcodes, n=3):
             group[-1] = tag, i1, min(i2, i1 + n), j1, min(j2, j1 + n)
         yield group
 
-def hdf_diff(fromlines, tolines, context=None, tabwidth=8,
-             ignore_blank_lines=0, ignore_case=0, ignore_space_changes=0):
-    """
-    Return an array that is adequate for adding the the HDF data set for HTML
-    rendering of the differences.
+def hdf_diff(*args, **kwargs):
+    return diff_blocks(*args, **kwargs)
+
+def diff_blocks(fromlines, tolines, context=None, tabwidth=8,
+                ignore_blank_lines=0, ignore_case=0, ignore_space_changes=0):
+    """Return an array that is adequate for adding to the data dictionary
+
+    See the diff_div.html template.
     """
 
     type_map = {'replace': 'mod', 'delete': 'rem', 'insert': 'add',
@@ -150,27 +152,13 @@ def hdf_diff(fromlines, tolines, context=None, tabwidth=8,
                 for i in range(i2 - i1):
                     fromline, toline = fromlines[i1 + i], tolines[j1 + i]
                     (start, end) = _get_change_extent(fromline, toline)
-
-                    if start == 0 and end < 0:
-                        # Change at start of line
-                        fromlines[i1 + i] = '\0' + fromline[:end] + '\1' + \
-                                            fromline[end:]
-                        tolines[j1 + i] = '\0' + toline[:end] + '\1' + \
-                                          toline[end:]
-                    elif start > 0 and end == 0:
-                        # Change at end of line
-                        fromlines[i1 + i] = fromline[:start] + '\0' + \
-                                            fromline[start:] + '\1'
-                        tolines[j1 + i] = toline[:start] + '\0' + \
-                                          toline[start:] + '\1'
-                    elif start > 0 and end < 0:
-                        # Change somewhere in the middle
-                        fromlines[i1 + i] = fromline[:start] + '\0' + \
-                                            fromline[start:end] + '\1' + \
-                                            fromline[end:]
-                        tolines[j1 + i] = toline[:start] + '\0' + \
-                                          toline[start:end] + '\1' + \
-                                          toline[end:]
+                    if start != 0 or end != 0:
+                        last = end+len(fromline)
+                        fromlines[i1+i] = fromline[:start] + '\0' + fromline[start:last] + \
+                                       '\1' + fromline[last:]
+                        last = end+len(toline)
+                        tolines[j1+i] = toline[:start] + '\0' + toline[start:last] + \
+                                     '\1' + toline[last:]
             yield tag, i1, i2, j1, j2
 
     changes = []
@@ -181,33 +169,37 @@ def hdf_diff(fromlines, tolines, context=None, tabwidth=8,
         last_tag = None
         for tag, i1, i2, j1, j2 in markup_intraline_changes(group):
             if tag != last_tag:
-                blocks.append({'type': type_map[tag], 'base.offset': i1,
-                               'base.lines': [], 'changed.offset': j1,
-                               'changed.lines': []})
+                blocks.append({'type': type_map[tag],
+                               'base': {'offset': i1, 'lines': []},
+                               'changed': {'offset': j1, 'lines': []}})
             if tag == 'equal':
                 for line in fromlines[i1:i2]:
                     line = line.expandtabs(tabwidth)
                     line = space_re.sub(htmlify, escape(line, quotes=False))
-                    blocks[-1]['base.lines'].append(Markup(line))
+                    blocks[-1]['base']['lines'].append(Markup(unicode(line)))
                 for line in tolines[j1:j2]:
                     line = line.expandtabs(tabwidth)
                     line = space_re.sub(htmlify, escape(line, quotes=False))
-                    blocks[-1]['changed.lines'].append(Markup(line))
+                    blocks[-1]['changed']['lines'].append(Markup(unicode(line)))
             else:
                 if tag in ('replace', 'delete'):
                     for line in fromlines[i1:i2]:
-                        line = line.expandtabs(tabwidth)
-                        line = escape(line, quotes=False).replace('\0', '<del>') \
-                                                         .replace('\1', '</del>')
-                        blocks[-1]['base.lines'].append(Markup(space_re.sub(htmlify,
-                                                                            line)))
+                        line = expandtabs(line, tabwidth, '\0\1')
+                        line = escape(line, quotes=False)
+                        line = '<del>'.join([space_re.sub(htmlify, seg)
+                                             for seg in line.split('\0')])
+                        line = line.replace('\1', '</del>')
+                        blocks[-1]['base']['lines'].append(
+                            Markup(unicode(line)))
                 if tag in ('replace', 'insert'):
                     for line in tolines[j1:j2]:
-                        line = line.expandtabs(tabwidth)
-                        line = escape(line, quotes=False).replace('\0', '<ins>') \
-                                                         .replace('\1', '</ins>')
-                        blocks[-1]['changed.lines'].append(Markup(space_re.sub(htmlify,
-                                                                               line)))
+                        line = expandtabs(line, tabwidth, '\0\1')
+                        line = escape(line, quotes=False)
+                        line = '<ins>'.join([space_re.sub(htmlify, seg)
+                                             for seg in line.split('\0')])
+                        line = line.replace('\1', '</ins>')
+                        blocks[-1]['changed']['lines'].append(
+                            Markup(unicode(line)))
         changes.append(blocks)
     return changes
 
@@ -217,6 +209,8 @@ def unified_diff(fromlines, tolines, context=None, ignore_blank_lines=0,
                            ignore_space_changes)
     for group in _group_opcodes(opcodes, context):
         i1, i2, j1, j2 = group[0][1], group[-1][2], group[0][3], group[-1][4]
+        if i1 == 0 and i2 == 0:
+            i1, i2 = -1, -1 # support for 'A'dd changes
         yield '@@ -%d,%d +%d,%d @@' % (i1 + 1, i2 - i1, j1 + 1, j2 - j1)
         for tag, i1, i2, j1, j2 in group:
             if tag == 'equal':
@@ -231,7 +225,9 @@ def unified_diff(fromlines, tolines, context=None, ignore_blank_lines=0,
                         yield '+' + line
 
 def get_diff_options(req):
-
+    options_data = {}
+    data = {'options': options_data}
+    
     def get_bool_option(name, default=0):
         pref = int(req.session.get('diff_' + name, default))
         arg = int(req.args.has_key(name))
@@ -245,7 +241,7 @@ def get_diff_options(req):
     style = req.args.get('style', pref)
     if req.args.has_key('update') and style != pref:
         req.session['diff_style'] = style
-    req.hdf['diff.style'] = style
+    data['style'] = style
 
     pref = int(req.session.get('diff_contextlines', 2))
     try:
@@ -255,24 +251,21 @@ def get_diff_options(req):
     if req.args.has_key('update') and arg != pref:
         req.session['diff_contextlines'] = arg
     options = ['-U%d' % arg]
-    if arg >= 0:
-        req.hdf['diff.options.contextlines'] = arg
-    else:
-        req.hdf['diff.options.contextlines'] = 'all'
+    options_data['contextlines'] = arg
 
     arg = get_bool_option('ignoreblanklines')
     if arg:
         options.append('-B')
-    req.hdf['diff.options.ignoreblanklines'] = arg
+    options_data['ignoreblanklines'] = arg
 
     arg = get_bool_option('ignorecase')
     if arg:
         options.append('-i')
-    req.hdf['diff.options.ignorecase'] = arg
+    options_data['ignorecase'] = arg
 
     arg = get_bool_option('ignorewhitespace')
     if arg:
         options.append('-b')
-    req.hdf['diff.options.ignorewhitespace'] = arg
+    options_data['ignorewhitespace'] = arg
 
-    return (style, options)
+    return (style, options, data)
