@@ -1,6 +1,7 @@
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2006 Edgewall Software
+# Copyright (C) 2003-2008 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005-2006 Matthew Good <trac@matt-good.net>
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
@@ -18,7 +19,7 @@
 #         Matthew Good <trac@matt-good.net>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-import errno
+import pkg_resources
 import os
 import sys
 from SocketServer import ThreadingMixIn
@@ -151,7 +152,7 @@ def main():
                       help='parent directory of the project environments')
     parser.add_option('--base-path', action='store', type='string', # XXX call this url_base_path?
                       dest='base_path',
-                      help='base path')
+                      help='the initial portion of the request URL\'s "path"')
 
     parser.add_option('-r', '--auto-reload', action='store_true',
                       dest='autoreload',
@@ -168,9 +169,13 @@ def main():
         parser.add_option('--pidfile', action='store',
                           dest='pidfile',
                           help='When daemonizing, file to which to write pid')
+        parser.add_option('--umask', action='store', type='int', dest='umask',
+                          metavar='MASK',
+                          help='When daemonizing, file mode creation mask '
+                          'to use (default 022)')
 
     parser.set_defaults(port=None, hostname='', base_path='', daemonize=False,
-                        protocol='http')
+                        protocol='http', umask=022)
     options, args = parser.parse_args()
 
     if not args and not options.env_parent_dir:
@@ -183,6 +188,9 @@ def main():
         elif len(args) > 1:
             parser.error('the --single-env option cannot be used with '
                          'more than one enviroment')
+    if options.daemonize and options.autoreload:
+        parser.error('the --auto-reload option cannot be used with '
+                     '--daemonize')
 
     if options.port is None:
         options.port = {
@@ -192,15 +200,6 @@ def main():
         }[options.protocol]
     server_address = (options.hostname, options.port)
 
-    # autoreload doesn't work when daemonized and using relative paths
-    if options.daemonize and options.autoreload:
-        for path in args + [options.env_parent_dir, options.pidfile]:
-            if path and not os.path.isabs(path):
-                parser.error('"%s" is not an absolute path.\n\n'
-                             'when using both --auto-reload and --daemonize '
-                             'all path arguments must be absolute'
-                             % path)
-
     # relative paths don't work when daemonized
     args = [os.path.abspath(a) for a in args]
     if options.env_parent_dir:
@@ -225,6 +224,13 @@ def main():
         def serve():
             httpd = TracHTTPServer(server_address, wsgi_app,
                                    options.env_parent_dir, args)
+            print 'Server starting in PID %i.' % os.getpid()
+            addr, port = server_address
+            if not addr or addr == '0.0.0.0':
+                print 'Serving on 0.0.0.0:%s view at http://127.0.0.1:%s/%s' \
+                       % (port, port, base_path)
+            else:
+                print 'Serving on http://%s:%s/%s' % (addr, port, base_path)
             httpd.serve_forever()
     elif options.protocol in ('scgi', 'ajp'):
         def serve():
@@ -234,39 +240,9 @@ def main():
             sys.exit(ret and 42 or 0) # if SIGHUP exit with status 42
 
     try:
-        if os.name == 'posix':
-            if options.pidfile:
-                options.pidfile = os.path.abspath(options.pidfile)
-                if os.path.exists(options.pidfile):
-                    pidfile = open(options.pidfile)
-                    try:
-                        pid = int(pidfile.read())
-                    finally:
-                        pidfile.close()
-
-                    try:
-                        # signal the process to see if it is still running
-                        os.kill(pid, 0)
-                    except OSError, e:
-                        if e.errno != errno.ESRCH:
-                            raise
-                    else:
-                        sys.exit("tracd is already running with pid %s" % pid)
-                realserve = serve
-                def serve():
-                    try:
-                        pidfile = open(options.pidfile, 'w')
-                        try:
-                            pidfile.write(str(os.getpid()))
-                        finally:
-                            pidfile.close()
-                        realserve()
-                    finally:
-                       if os.path.exists(options.pidfile):
-                           os.remove(options.pidfile)
-
-            if options.daemonize:
-                daemon.daemonize()
+        if options.daemonize:
+            daemon.daemonize(pidfile=options.pidfile, progname='tracd',
+                             umask=options.umask)
 
         if options.autoreload:
             def modification_callback(file):
@@ -282,4 +258,5 @@ def main():
         pass
 
 if __name__ == '__main__':
+    pkg_resources.require('Trac==%s' % VERSION)
     main()
