@@ -1,33 +1,43 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2005 Edgewall Software
-# Copyright (C) 2003-2005 Jonas Borgstrm <jonas@edgewall.com>
+# Copyright (C) 2003-2008 Edgewall Software
+# Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution. The terms
-# are also available at http://trac.edgewall.com/license.html.
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
 #
 # This software consists of voluntary contributions made by many
 # individuals. For the exact contribution history, see the revision
-# history and logs, available at http://projects.edgewall.com/trac/.
+# history and logs, available at http://trac.edgewall.org/log/.
 #
-# Author: Jonas Borgstrm <jonas@edgewall.com>
+# Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import generators
 import time
+from datetime import datetime
 
 from trac.core import *
+from trac.resource import Resource
+from trac.util.datefmt import utc, to_timestamp
+from trac.util.translation import _
 from trac.wiki.api import WikiSystem
 
 
 class WikiPage(object):
     """Represents a wiki page (new or existing)."""
 
+    realm = 'wiki'
+
     def __init__(self, env, name=None, version=None, db=None):
         self.env = env
+        if isinstance(name, Resource):
+            self.resource = name
+            name = self.resource.id
+        else:
+            self.resource = Resource('wiki', name, version)
         self.name = name
         if name:
             self._fetch(name, version, db)
@@ -43,22 +53,28 @@ class WikiPage(object):
             db = self.env.get_db_cnx()
         cursor = db.cursor()
         if version:
-            cursor.execute("SELECT version,text,readonly FROM wiki "
+            cursor.execute("SELECT version,time,author,text,comment,readonly "
+                           "FROM wiki "
                            "WHERE name=%s AND version=%s",
                            (name, int(version)))
         else:
-            cursor.execute("SELECT version,text,readonly FROM wiki "
+            cursor.execute("SELECT version,time,author,text,comment,readonly "
+                           "FROM wiki "
                            "WHERE name=%s ORDER BY version DESC LIMIT 1",
                            (name,))
         row = cursor.fetchone()
         if row:
-            version,text,readonly = row
+            version,time,author,text,comment,readonly = row
             self.version = int(version)
+            self.author = author
+            self.time = datetime.fromtimestamp(time, utc)
             self.text = text
+            self.comment = comment
             self.readonly = readonly and int(readonly) or 0
         else:
             self.version = 0
-            self.text = ''
+            self.text = self.comment = self.author = ''
+            self.time = None
             self.readonly = 0
 
     exists = property(fget=lambda self: self.version > 0)
@@ -71,7 +87,6 @@ class WikiPage(object):
         else:
             handle_ta = False
 
-        page_deleted = False
         cursor = db.cursor()
         if version is None:
             # Delete a wiki page completely
@@ -90,16 +105,21 @@ class WikiPage(object):
         if not self.exists:
             from trac.attachment import Attachment
             # Delete orphaned attachments
-            for attachment in Attachment.select(self.env, 'wiki', self.name, db):
-                attachment.delete(db)
-
-            # Let change listeners know about the deletion
-            for listener in WikiSystem(self.env).change_listeners:
-                listener.wiki_page_deleted(self)
+            Attachment.delete_all(self.env, 'wiki', self.name, db)
 
         if handle_ta:
             db.commit()
 
+        # Let change listeners know about the deletion
+        if not self.exists:
+            for listener in WikiSystem(self.env).change_listeners:
+                listener.wiki_page_deleted(self)
+        else:
+            for listener in WikiSystem(self.env).change_listeners:
+                if hasattr(listener, 'wiki_page_version_deleted'):
+                    listener.wiki_page_version_deleted(self)
+
+
     def save(self, author, comment, remote_addr, t=None, db=None):
         if not db:
             db = self.env.get_db_cnx()
@@ -108,21 +128,22 @@ class WikiPage(object):
             handle_ta = False
 
         if t is None:
-            t = time.time()
+            t = datetime.now(utc)
 
         if self.text != self.old_text:
             cursor = db.cursor()
-            cursor.execute("INSERT INTO WIKI (name,version,time,author,ipnr,"
+            cursor.execute("INSERT INTO wiki (name,version,time,author,ipnr,"
                            "text,comment,readonly) VALUES (%s,%s,%s,%s,%s,%s,"
-                           "%s,%s)", (self.name, self.version + 1, t, author,
-                           remote_addr, self.text, comment, self.readonly))
+                           "%s,%s)", (self.name, self.version + 1,
+                                      to_timestamp(t), author, remote_addr,
+                                      self.text, comment, self.readonly))
             self.version += 1
         elif self.readonly != self.old_readonly:
             cursor = db.cursor()
             cursor.execute("UPDATE wiki SET readonly=%s WHERE name=%s",
                            (self.readonly, self.name))
         else:
-            raise TracError('Page not modified')
+            raise TracError(_('Page not modified'))
 
         if handle_ta:
             db.commit()
@@ -144,5 +165,6 @@ class WikiPage(object):
         cursor.execute("SELECT version,time,author,comment,ipnr FROM wiki "
                        "WHERE name=%s AND version<=%s "
                        "ORDER BY version DESC", (self.name, self.version))
-        for version,time,author,comment,ipnr in cursor:
+        for version,ts,author,comment,ipnr in cursor:
+            time = datetime.fromtimestamp(ts, utc)
             yield version,time,author,comment,ipnr
